@@ -16,7 +16,6 @@ from trytond.wizard import (Wizard, StateView, StateTransition, StateReport,
     Button)
 from trytond.tools import grouped_slice
 from trytond.modules.company import CompanyReport
-from trytond import backend
 
 __all__ = ['Asset', 'AssetLine', 'AssetUpdateMove',
     'CreateMovesStart', 'CreateMoves',
@@ -186,27 +185,31 @@ class Asset(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'run': {
                     'invisible': Eval('state') != 'draft',
+                    'depends': ['state'],
                     },
                 'close': {
                     'invisible': Eval('state') != 'running',
+                    'depends': ['state'],
                     },
                 'create_lines': {
                     'invisible': (Eval('lines', [])
                         | (Eval('state') != 'draft')),
+                    'depends': ['state'],
                     },
                 'clear_lines': {
                     'invisible': (~Eval('lines', [0])
                         | (Eval('state') != 'draft')),
+                    'depends': ['state'],
                     },
                 'update': {
                     'invisible': Eval('state') != 'running',
+                    'depends': ['state'],
                     },
                 })
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table_h = TableHandler(cls, module_name)
+        table_h = cls.__table_handler__(module_name)
 
         # Migration from 3.8: rename reference into number
         if table_h.column_exist('reference'):
@@ -327,6 +330,10 @@ class Asset(Workflow, ModelSQL, ModelView):
         Returns all the remaining dates at which asset depreciation movement
         will be issued.
         """
+        pool = Pool()
+        Config = pool.get('account.configuration')
+        config = Config(1)
+
         start_date = max([self.start_date] + [l.date for l in self.lines])
         delta = relativedelta.relativedelta(self.end_date, start_date)
         # dateutil >= 2.0 has replace __nonzero__ by __bool__ which doesn't
@@ -335,10 +342,11 @@ class Asset(Workflow, ModelSQL, ModelView):
             return [self.end_date]
         if self.frequency == 'monthly':
             rule = rrule.rrule(rrule.MONTHLY, dtstart=self.start_date,
-                bymonthday=-1)
+                bymonthday=int(config.asset_bymonthday))
         elif self.frequency == 'yearly':
             rule = rrule.rrule(rrule.YEARLY, dtstart=self.start_date,
-                bymonth=12, bymonthday=-1)
+                bymonth=int(config.asset_bymonth),
+                bymonthday=int(config.asset_bymonthday))
         dates = [d.date()
             for d in rule.between(date2datetime(start_date),
                 date2datetime(self.end_date))]
@@ -413,7 +421,7 @@ class Asset(Workflow, ModelSQL, ModelView):
             if residual_value > 0 and asset_line is not None:
                 asset_line.depreciation += residual_value
                 asset_line.accumulated_depreciation += residual_value
-        for asset_line in amounts.itervalues():
+        for asset_line in amounts.values():
             asset_line.actual_value = (self.value -
                 asset_line.accumulated_depreciation)
         return amounts
@@ -427,7 +435,7 @@ class Asset(Workflow, ModelSQL, ModelView):
 
         lines = []
         for asset in assets:
-            for date, line in asset.depreciate().iteritems():
+            for date, line in asset.depreciate().items():
                 line.asset = asset.id
                 line.date = date
                 lines.append(line)
@@ -460,16 +468,17 @@ class Asset(Workflow, ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
 
         period_id = Period.find(self.company.id, line.date)
-        expense_line = MoveLine(
-            credit=0,
-            debit=line.depreciation,
-            account=self.product.account_expense_used,
-            )
-        depreciation_line = MoveLine(
-            debit=0,
-            credit=line.depreciation,
-            account=self.product.account_depreciation_used,
-            )
+        with Transaction().set_context(date=line.date):
+            expense_line = MoveLine(
+                credit=0,
+                debit=line.depreciation,
+                account=self.product.account_expense_used,
+                )
+            depreciation_line = MoveLine(
+                debit=0,
+                credit=line.depreciation,
+                account=self.product.account_depreciation_used,
+                )
 
         return Move(
             company=self.company,
@@ -518,7 +527,7 @@ class Asset(Workflow, ModelSQL, ModelView):
         date = Date.today()
         period_id = Period.find(self.company.id, date)
         if self.supplier_invoice_line:
-            account_asset = self.supplier_invoice_line.account
+            account_asset = self.supplier_invoice_line.account.current()
         else:
             account_asset = self.product.account_asset_used
 
@@ -610,11 +619,12 @@ class Asset(Workflow, ModelSQL, ModelView):
     def copy(cls, assets, default=None):
         if default is None:
             default = {}
-        default = default.copy()
+        else:
+            default = default.copy()
         default.setdefault('lines', [])
-        default.setdefault('state', 'draft')
         default.setdefault('number', None)
         default.setdefault('supplier_invoice_line', None)
+        default.setdefault('move')
         return super(Asset, cls).copy(assets, default=default)
 
     @classmethod
@@ -629,15 +639,16 @@ class AssetLine(ModelSQL, ModelView):
     'Asset Line'
     __name__ = 'account.asset.line'
     asset = fields.Many2One('account.asset', 'Asset', required=True,
-        ondelete='CASCADE')
-    date = fields.Date('Date')
+        ondelete='CASCADE', readonly=True)
+    date = fields.Date('Date', readonly=True)
     depreciation = fields.Numeric('Depreciation',
         digits=(16, Eval('_parent_asset', {}).get('currency_digits', 2)),
-        required=True)
-    acquired_value = fields.Numeric('Acquired Value')
-    depreciable_basis = fields.Numeric('Depreciable Basis')
-    actual_value = fields.Numeric('Actual Value')
-    accumulated_depreciation = fields.Numeric('Accumulated Depreciation')
+        required=True, readonly=True)
+    acquired_value = fields.Numeric('Acquired Value', readonly=True)
+    depreciable_basis = fields.Numeric('Depreciable Basis', readonly=True)
+    actual_value = fields.Numeric('Actual Value', readonly=True)
+    accumulated_depreciation = fields.Numeric(
+        'Accumulated Depreciation', readonly=True)
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
 
     @classmethod
@@ -897,16 +908,15 @@ class AssetDepreciationTable(CompanyReport):
 
             @cached_property
             def asset_lines(self):
-                return filter(
-                    lambda l: self.start_date < l.date <= self.end_date,
-                    self.asset.lines)
+                return [l for l in self.asset.lines
+                    if self.start_date < l.date <= self.end_date]
 
             @cached_property
             def update_lines(self):
                 filter_ = lambda l: (l.account.kind == 'other'
                     and self.start_date < l.move.date <= self.end_date)
-                return filter(filter_,
-                    (l for m in self.asset.update_moves for l in m.lines))
+                return list(filter(filter_,
+                    (l for m in self.asset.update_moves for l in m.lines)))
 
             @cached_property
             def start_fixed_value(self):
